#!/usr/bin/python
# -*- Encoding: utf-8 -*-

json_index = r'/mnt/data/data/com.onyx.android.scribbler/files/notes.json'
json_index_bak = r'/mnt/data/data/com.onyx.android.scribbler/files/notes.json.bak'
scribble_glob = '/mnt/data/data/com.onyx.android.data/databases/onyx.scribble.*.db'

"""
[{"current_page":1,"description":"","md5":"f41dbcd8eb82a1a28a980dbc0af8c50b","page_count":3,"time":1458853238581,"title":"2016-03-24 22:00:38"},{"description":"","md5":"d1f76727365193f8e604ba82970554ec","page_count":4,"time":1460671556825,"title":"2016-04-15 00:05:53"},{"description":"","md5":"0ecd1a01c5cd4289bd5970624b11373a","page_count":2,"time":1460933330981,"title":"2016-04-18 00:48:49"}]
"""
import json
import glob
import re
import os
import time
import sqlite3
import uuid

def fix_json_index():
    data = json.load(open(json_index, 'r'))
    existing_md5 = set()

    for e in data:
        existing_md5.add(e['md5'])

    for db_file in glob.glob(scribble_glob):
        m = re.search(r"onyx\.scribble\.([0-9a-f]+)\.db", db_file)
        if m:
            md5 = m.group(1)
            missing = md5 not in existing_md5
            remove = False
            conn = sqlite3.connect(db_file)
            c = conn.cursor()
            c.execute("select count(*) as n_records from library_scribble;")
            n_records = c.fetchone()[0]
            c.execute("select distinct Application from library_scribble;")
            res = c.fetchone()
            if res is None:
                application = None
            else:
                application = res[0]
            conn.close()
            if n_records == 0:
                print "Database for %s has no data" % md5
                missing = False
                remove = md5 in existing_md5
            if application <> "com.onyx.android.scribbler":
                print "Database for %s contains data for %s, not Scribbler" % (md5, application)
                missing = False
                remove = md5 in existing_md5
            if missing:
                print md5, "missing in json index, adding ..."
                modified = os.path.getmtime(db_file)
                e = {
                    "current_page": 1,
                    "description": "",
                    "md5": md5,
                    "page_count": 1,  # unknown, hope this works
                    "time": int(modified * 1000),
                    "title": time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime(modified))
                    }
                data.append(e)
                #print json.dumps(e)
            elif remove:
                print "Removing %s from json index ..." % md5
                data = [e for e in data if e['md5'] <> md5]
    st = os.stat(json_index)
    os.rename(json_index, json_index_bak)
    json.dump(data, open(json_index, 'w'))
    os.chown(json_index, st.st_uid, st.st_gid)

def fix_missing_uniqueId():
    for db_file in glob.glob(scribble_glob):
        conn = sqlite3.connect(db_file)
        c = conn.cursor()
        c.execute("select * from library_scribble limit 1;")
        field_names = [i[0] for i in c.description]
        if 'uniqueId' not in field_names:
            print "library_scribble.uniqueId missing in %s, adding ..." % db_file
            c.execute("alter table library_scribble add column uniqueId TEXT;")
            print "generating uuids ..."
            c.execute("select _id from library_scribble;")
            for row in c.fetchall():
                c.execute("update library_scribble set uniqueId=? where _id=?",
                    (str(uuid.uuid4()), row[0]))
            conn.commit()
            print "done."
        conn.close()
    

fix_json_index()
fix_missing_uniqueId()
